@@ -124,7 +124,7 @@ yh_init (YelpHtml *view)
 
         priv->doc         = html_document_new ();
         priv->connections = NULL;
-        priv->base_uri    = NULL;
+        priv->base_uri    = g_strdup ("");
 
         html_view_set_document (HTML_VIEW (view), priv->doc);
         
@@ -249,6 +249,7 @@ yh_url_requested_cb (HtmlDocument *doc,
         sdata         = g_new0 (StreamData, 1);
 	sdata->view   = view;
 	sdata->stream = stream;
+
 	
         priv->connections = g_slist_prepend (priv->connections, sdata);
 
@@ -266,25 +267,6 @@ yh_url_requested_cb (HtmlDocument *doc,
 	html_stream_set_cancel_func (stream, yh_stream_cancel, sdata);
 }
 
-/* static void */
-/* kill_old_connections (HtmlDocument *doc) */
-/* { */
-/* 	GSList *connection_list, *tmp; */
-
-/* 	tmp = connection_list = g_object_get_data (G_OBJECT (doc), "connection_list"); */
-/* 	while(tmp) { */
-
-/* 		StreamData *sdata = (StreamData *)tmp->data; */
-/* 		gnome_vfs_async_cancel (sdata->handle); */
-/* 		free_stream_data (sdata, FALSE); */
-
-/* 		tmp = tmp->next; */
-/* 	} */
-
-/* 	g_object_set_data (G_OBJECT (doc), "connection_list", NULL); */
-/* 	g_slist_free (connection_list); */
-/* } */
-
 static void
 yh_stream_cancel (HtmlStream *stream, 
 		  gpointer    user_data, 
@@ -312,8 +294,6 @@ yh_free_stream_data (StreamData *sdata, gboolean remove)
 		priv->connections = g_slist_remove (priv->connections, sdata);
 	}
 
-	g_print ("Out of stream_data\n");
-
 	if (sdata->anchor) {
 		g_free (sdata->anchor);
 	}
@@ -370,8 +350,11 @@ yelp_html_new (void)
         return GTK_WIDGET (view);
 }
 
+
 void
-yelp_html_open_section (YelpHtml *view, const YelpSection *section)
+yelp_html_open_uri (YelpHtml    *view, 
+		    const gchar *str_uri,
+		    const gchar *reference)
 {
         YelpHtmlPriv *priv;
         StreamData   *sdata;
@@ -381,11 +364,11 @@ yelp_html_open_section (YelpHtml *view, const YelpSection *section)
         d(puts(G_GNUC_FUNCTION));
 	
 	g_return_if_fail (YELP_IS_HTML (view));
-	g_return_if_fail (section != NULL);
+	g_return_if_fail (str_uri != NULL);
 
         priv = view->priv;
 
-	d(g_print ("Trying to open: %s\n", section->uri));
+	d(g_print ("Trying to open: %s\n", str_uri));
 	
         html_document_clear (priv->doc);
         html_document_open_stream (priv->doc, "text/html");
@@ -393,11 +376,10 @@ yelp_html_open_section (YelpHtml *view, const YelpSection *section)
 	gtk_adjustment_set_value (
 		gtk_layout_get_vadjustment (GTK_LAYOUT (view)), 0);
 
-	if (priv->base_uri) {
+	if (strcmp (priv->base_uri, str_uri)) {
 		g_free (priv->base_uri);
+		priv->base_uri = g_strdup (str_uri);
 	}
-	
-        priv->base_uri = g_strdup (section->uri);
 
 	sdata          = g_new0 (StreamData, 1);
 	sdata->view    = view;
@@ -406,18 +388,16 @@ yelp_html_open_section (YelpHtml *view, const YelpSection *section)
 	
 	priv->connections = g_slist_prepend (priv->connections, sdata);
 
-	if (section->reference) {
- 		gchar *tmp_uri = g_strconcat (section->uri, 
-					      section->reference,
-					      NULL);
+	if (reference) {
+ 		gchar *tmp_uri = g_strconcat (str_uri, reference, NULL);
 		uri = gnome_vfs_uri_new (tmp_uri);
 		g_free (tmp_uri);
 	} else {
-		uri = gnome_vfs_uri_new (section->uri);
+		uri = gnome_vfs_uri_new (str_uri);
 	}
 
-	if (section->reference) {
-		sdata->anchor = g_strdup (section->reference);
+	if (reference) {
+		sdata->anchor = g_strdup (reference);
 	} else if (uri) {
 		fragment = gnome_vfs_uri_get_fragment_identifier (uri);
 		if (fragment) {
