@@ -53,9 +53,11 @@ struct _YelpWindowClass
 };
 
 GType            yelp_window_get_type        (void);
-GtkWidget *      yelp_window_new             (GNode         *doc_tree);
+GtkWidget *      yelp_window_new             (GNode         *doc_tree,
+					      GList         *index);
 
 void             yelp_window_open_uri        (YelpWindow    *window,
 					      const gchar   *str_uri);
+const gchar *    yelp_window_get_current_uri (YelpWindow    *window);
 
 #endif /* __YELP_WINDOW_H__ */
